import java.awt.*;
import java.awt.image.BufferedImage;

import com.golden.gamedev.Game;
import com.golden.gamedev.object.AnimatedSprite;
import com.golden.gamedev.object.PlayField;
import com.golden.gamedev.object.GameFont;
import com.golden.gamedev.object.background.ColorBackground;

/**
 * Example of animated sprites.<br>
 * Creates a playfield and add 20 balls (animated sprites).<br>
 * The size of each ball is 31x31 pixels.
 */
public class Sample1 extends Game {
	private PlayField 	playfield;	// the playfield
	private GameFont 	f;			// font for draw frame per second

	public void initResources() {
		f = fontManager.getFont(new Font("Verdana", Font.BOLD, 15));

		// init the playfield, with black background
		playfield = new PlayField(new ColorBackground(Color.BLACK));

		// loading images
		// ball image (4 rows, 3 columns, no masking)
		BufferedImage[] image = getImages("ball.gif", 4, 3, false);

	    // creates 20 balls (4 lines, 5 columns)
		for (int i=0;i < 4;i++) {
			for (int j=0;j < 5;j++) {
				// x and y coordinate
				int x = (j*62) + 15,
					y = (i*62) + 15;

				// random animation delay
				int frame = getRandom(0, 5);

				// creates the ball with specified image and position
				AnimatedSprite ball = new AnimatedSprite(image, x, y);
				ball.getAnimationTimer().setDelay(frame);
				ball.setAnimate(true);
				ball.setLoopAnim(true);

				// add the ball into playfield
				playfield.add(ball);
			}
		}
	}

	public void update() {
		playfield.update();
	}

	public void render(Graphics2D g) {
		playfield.render(g);

		// render frame per second
		g.setColor(Color.YELLOW);
		f.drawString(g, "FPS = "+getCurrentFPS()+" / "+getRequestedFPS(), 10, 2);
	}

	public static void main(String[] args) {
		Game game = new Sample1();
		game.setup(new Dimension(310, 248), false);
		game.start();
	}

}