import java.awt.*;
import java.awt.image.BufferedImage;

import com.golden.gamedev.Game;
import com.golden.gamedev.object.AnimatedSprite;
import com.golden.gamedev.object.PlayField;
import com.golden.gamedev.object.Background;
import com.golden.gamedev.object.GameFont;
import com.golden.gamedev.object.background.ColorBackground;

/**
 * Example of extending Sprite class.
 * The extended sprite will be bounced every time collided with screen bounds.
 */
public class Sample2 extends Game {
	private PlayField 	playfield;	// the playfield
	private GameFont 	f;			// font for draw frame per second

	public void initResources() {
		f = fontManager.getFont(new Font("Verdana", Font.BOLD, 15));

		// creates the playfield and ball group
		playfield = new PlayField(new ColorBackground(Color.BLACK));

		// loading image
		BufferedImage[] image = getImages("ball.gif", 4, 3, false);

	    // creates 20 balls (4 lines, 5 columns)
		for (int i=0;i < 4;i++) {
			for (int j=0;j < 5;j++) {
				int x = (j*62) + 15,
					y = (i*62) + 15;
				playfield.add(new BouncingSprite(image, x, y));
			}
		}
	}

	public void update() {
		playfield.update();
	}
	public void render(Graphics2D g) {
		playfield.render(g);

		// render frame per second
		g.setColor(Color.YELLOW);
		f.drawString(g, "FPS = "+getCurrentFPS()+" / "+getRequestedFPS(), 10, 2);
	}

	public static void main(String[] args) {
		Game game = new Sample2();
		game.setup(new Dimension(310, 248), false);
		game.start();
	}


	/**
	 * The bouncing animated sprite.
	 * This sprite will bounce when collide with screen boundary.
	 */
	class BouncingSprite extends AnimatedSprite {

		public BouncingSprite(BufferedImage[] image, int x, int y) {
			super(image,x,y);

			setAnimate(true);
			setLoopAnim(true);
			getAnimationTimer().setDelay(getRandom(0, 5));

			speedX = 1;
			speedY = 1;
		}

		public void update() {
			super.update();

			Background backgr = getBackground();

			// bounds collision check
			if (getX() < 0 || getX() + getWidth() > backgr.getWidth()) {
				// left-right bounds
				speedX *= -1;
				moveX(speedX);
			}

			if (getY() < 0 || getY() + getHeight() > backgr.getHeight()) {
				// top-bottom bounds
				speedY *= -1;
				moveY(speedY);
			}
		}
	}

}