import java.awt.*;
import java.awt.image.*;

import com.golden.gamedev.*;
import com.golden.gamedev.object.*;
import com.golden.gamedev.object.background.ColorBackground;

/**
 * Example of using collision event.<br>
 * All balls can be collided each other.
 */
public class Sample3 extends Game {
	private PlayField 	playfield;	// the playfield
	private SpriteGroup	BALL;		// the ball group
	private GameFont 	f;			// font for draw frame per second

	public void initResources() {
		f = fontManager.getFont(new Font("Verdana", Font.BOLD, 15));

		// creates the playfield
		playfield = new PlayField(new ColorBackground(Color.BLACK));

		BufferedImage[] image = getImages("ball.gif", 4, 3, false);
		BALL = new SpriteGroup("Ball"); // creates ball group
		for (int i=0;i < 2;i++) {
			for (int j=0;j < 3;j++) {
				int x = (j*124) + 15,
					y = (i*124) + 15;
				// add all ball into ball group
				BALL.add(new BouncingSprite(image, x, y));
			}
		}

		// the trigger ball :-)
		Sprite movingBall = new BouncingSprite(image, 75, 75);
		movingBall.setSpeed(2, 2);
		BALL.add(movingBall);

		// add ball group into playfield
		playfield.addGroup(BALL);

		// set collision event between ball and ball
		playfield.addCollisionGroup(BALL, BALL, new CollisionGroup() {
			public void collided(Sprite s1, Sprite s2) {
				// revert to last position before collision
				revertPosition1();
				revertPosition2();

				if (getCollisionSide() == LEFT_RIGHT_COLLISION ||
					getCollisionSide() == RIGHT_LEFT_COLLISION) {
					// side collision
					s1.speedX = -s1.speedX;
					s2.speedX = -s2.speedX;

					if (s1.speedX == 0) {
						s1.speedX = -s2.speedX;
					}
					if (s2.speedX == 0) {
						s2.speedX = -s1.speedX;
					}

				} else {
					// top-bottom collision
					s1.speedY = -s1.speedY;
					s2.speedY = -s2.speedY;

					if (s1.speedY == 0) {
						s1.speedY = -s2.speedY;
					}
					if (s2.speedY == 0) {
						s2.speedY = -s1.speedY;
					}
				}
			}
		} );
	}

	public void update() {
		playfield.update();
	}

	public void render(Graphics2D g) {
		playfield.render(g);

		// render frame per second
		g.setColor(Color.YELLOW);
		f.drawString(g, "FPS = "+getCurrentFPS()+" / "+getRequestedFPS(), 10, 2);
	}

	/**
	 * The bouncing sprite, extend from animated sprite.
	 * This sprite will bounce when collide with screen boundary.
	 */
	class BouncingSprite extends AnimatedSprite {

		public BouncingSprite(BufferedImage[] image, int x, int y) {
			super(image,x,y);

			setAnimate(true);
			setLoopAnim(true);
			getAnimationTimer().setDelay(getRandom(0, 4));
		}

		public void update() {
			super.update();

			// out of bounds check
			Background backgr = getBackground();
			if (getX() < 0 || getX() + getWidth() > backgr.getWidth()) {
				// left-right bounds
				speedX *= -1;
				moveX(speedX);
			}
			if (getY() < 0 || getY() + getHeight() > backgr.getHeight()) {
				// top-bottom bounds
				speedY *= -1;
				moveY(speedY);
			}
		}
	}

	public static void main(String[] args) {
		Game game = new Sample3();
		game.setup(new Dimension(310, 248), false);
		game.start();
	}

}