package gamrprj;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Line2D;
import java.awt.geom.Point2D;

/**
 * <p>
 * Title: 2D  ùķ̼
 * </p>
 * <p>
 * Description:  Ʈ 2D   ǽð  ùķ̼ ǥ ϰ ֽϴ.  帮ڸ,
 * Ͽ Ȩ  ǽðȭϴ  ǥԴϴ.
 * </p>
 * <p>
 * Copyright: Copyright (c) 2003
 * </p>
 * <p>
 * Company:
 * </p>
 * 
 * @author Jisung, AHn
 * @version 1.0
 */
public abstract class AbstractEntity implements Entity {

	static CollisionEngine _collisionEngine;
	static TranslateEngine _translateEngine;
	static long _entityCount;
	String _name;
	Shape _originalShape;
	Shape _transformedShape;
	AffineTransform _rotateTransform;
	AffineTransform _translateTransform;
	AffineTransform _scaleTransform;
	Point2D[] _points;
	Line2D[] _lines;
	boolean _isTransformed = false;
	int _direction = 0;
	int _oldDirection = 0;
	double _x = 0;
	double _oldX = 1;
	double _y = 0;
	double _oldY = 1;
	double _speed = 0;
	double _mass;
	int _life;
	double _xVector = 0;
	double _yVector = 0;
	double _zoom = 1D;
	double _oldZoom = 1D;
	static {
		if (_collisionEngine == null) {
			_collisionEngine = new CollisionEngine();
		}
		if (_translateEngine == null) {
			_translateEngine = new TranslateEngine();
		}
	}


	public AbstractEntity() {
		_name = "(Unnamed) Entity #" + _entityCount;
		_entityCount++;
	}


	public AbstractEntity(String name) {
		_entityCount++;
		_name = name;
	}


	public AbstractEntity(String name, double aX, double aY, int aDir, double aSpeed) {
		this(name);
		this._originalShape = getSelfShape();
		_x = aX;
		_y = aY;
		_direction = aDir;
		_oldDirection = _direction;
		_speed = aSpeed;
		calcVector();
		_rotateTransform = AffineTransform.getRotateInstance(_direction);
		_translateTransform = AffineTransform.getTranslateInstance(_x, _y);
		_scaleTransform = AffineTransform.getScaleInstance(_zoom, _zoom);
		_transformedShape = this.getTransformedShape();
		_points = _collisionEngine.getPointsOnShape(_originalShape);
		_lines = _collisionEngine.getLinesOnShape(_originalShape);
	}


	public AbstractEntity(String name, double aX, double aY, int aDir, double aSpeed, int aLife,
			double aMass) {
		this(name, aX, aY, aDir, aSpeed);
		_life = aLife;
		_mass = aMass;
	}


	public abstract Shape getSelfShape();


	//        Polygon p = new Polygon();
	//// p.addPoint( -15, -15);
	//        p.addPoint(15, -15);
	//        p.addPoint(10, 65);
	//        p.addPoint( -10, 65);
	//        return p;
	//    }
	public String getName() {
		return _name;
	}


	public Point2D getPosition() {
		return new Point2D.Double(_x, _y);
	}


	public double getX() {
		return _x;
	}


	public void setX(double aX) {
		_oldX = _x;
		_x += aX;
		_translateTransform.translate(aX, 0);
	}


	public double getY() {
		return _y;
	}


	public void setY(double aY) {
		_oldY = _y;
		_y += aY;
		_translateTransform.translate(0, aY);
	}


	public int getDirection() {
		return _oldDirection;
	}


	public void setDirection(int aDirection) {
		int temp = _oldDirection;
		temp += aDirection;
		if (temp >= 360) {
			temp -= 360;
		}
		if (temp < 0) {
			temp = 360 + temp;
		}
		_oldDirection = _direction;
		_direction = temp;
		double rad = _translateEngine.toRad(_direction);
		calcVector();
		_rotateTransform = AffineTransform.getRotateInstance(rad);
	}


	void calcVector() {
		int angle = _oldDirection;
		double cos = _translateEngine.cos(angle);
		double sin = _translateEngine.sin(angle);
		_xVector = -(_speed * sin);
		_yVector = _speed * cos;
	}


	public double getSpeed() {
		return _speed;
	}


	public void setSpeed(double aSpeed) {
		_speed = aSpeed;
	}


	public int getLife() {
		return _life;
	}


	public void setLife(int aLife) {
		_life = aLife;
	}


	public boolean isAlive() {
		return _life > 0;
	}


	public double getMass() {
		return _mass;
	}


	public void setMass(double aMass) {
		_mass = aMass;
	}


	public abstract void execute();


	//     {
	//        setX(_xVector);
	//        setY(_yVector);
	//        _isTransformed = false;
	//    }
	public double[] getVector() {
		return new double[]{_xVector, _yVector};
	}


	public double getXVector() {
		return _xVector;
	}


	public double getYVector() {
		return _yVector;
	}


	public double getXAccelation() {
		return 0;
	}


	public double getYAccelation() {
		return 0;
	}


	public Point2D[] getPoints() {
		return _points;
	}


	public Line2D[] getLines() {
		return _lines;
	}


	public Shape getOriginalShape() {
		return _originalShape;
	}


	public Shape getTransformedShape() {
		if (_isTransformed) {
			return _transformedShape;
		}
		if (_x == _oldX && _y == _oldY && _direction == _oldDirection) {
			return _transformedShape;
		}
		synchronized (_originalShape) {
			//        	if (_direction != _oldDirection) {
			_transformedShape = _rotateTransform.createTransformedShape(_originalShape);
			//        	}
			//        	if ( _x != _oldX || _y != _oldY ) {
			_transformedShape = _translateTransform.createTransformedShape(_transformedShape);
			//        	}
			//            if (_zoom != _oldZoom) {
			_transformedShape = _scaleTransform.createTransformedShape(_transformedShape);
			//            }
		}
		_points = _collisionEngine.getPointsOnShape(_transformedShape);
		_lines = _collisionEngine.getLinesOnShape(_transformedShape);
		_isTransformed = true;
		return _transformedShape;
	}


	public void zoom(double scale) {
		_oldZoom = _zoom;
		_scaleTransform.scale(scale, scale);
	}


	public AffineTransform getRotateTransform() {
		return _rotateTransform;
	}


	public AffineTransform getTranslateTransform() {
		return _translateTransform;
	}


	public AffineTransform getScaleTransform() {
		return _scaleTransform;
	}
}