/*
 * Created on 2004. 6. 7.
 *
 * All right reserved to Jisung, Ahn
 */

package net.narusas.game.eun1.units;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Rectangle;
import java.awt.geom.Line2D;
import java.util.ArrayList;

import net.narusas.game.eun1.MathUtil;
import net.narusas.game.framework.Unit;
import net.narusas.game.framework.util.EaseInOutInterpolation;

/**
 * 
 * 
 * @author Jisung, Ahn
 * 
 *  
 */
public class CircleUnit implements Unit {

	private int _x;
	private int _y;
	private int _radious;
	private Color _lineColor;
	private Color _fillColor;
	private float _bearing;
	private float _speed;
	private ArrayList _ways;
	private Point _nextWay;
	private boolean _isRotating;
	private float _nextAngle;
	private EaseInOutInterpolation _interpolation;

	public CircleUnit(int x, int y, int radious, float bearing, float speed,
			Color lineColor, Color fillColor) {
		_x = x;
		_y = y;
		_bearing = bearing;
		_speed = speed;
		_radious = radious;
		_lineColor = lineColor;
		_fillColor = fillColor;
		_ways = new ArrayList();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.eun1.Unit#getX()
	 */
	public int getX() {
		return _x;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.eun1.Unit#getY()
	 */
	public int getY() {
		return _y;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.eun1.Unit#getBounds()
	 */
	public Rectangle getBounds() {
		return new Rectangle(_x - _radious, _y - _radious, _radious * 2,
				_radious * 2);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.eun1.Unit#draw(java.awt.Graphics2D)
	 */
	public void draw(Graphics2D g2, Rectangle view) {
		if (!view.contains(getBounds())) {
			return;
		}
		int R = _radious * 2;
		int cx = _x - (int) view.getX();
		int cy = _y - (int) view.getY();
		g2.setColor(_fillColor);
		g2.fillOval(cx - _radious, cy - _radious, R, R);
		g2.setColor(_lineColor);
		g2.drawOval(cx - _radious, cy - _radious, R, R);
		float bearing = getBearing() + 270;
		int px = (int) (Math.cos(Math.toRadians(bearing)) * 40) + cx;
		int py = (int) (Math.sin(Math.toRadians(bearing)) * 40) + cy;
		g2.drawLine(cx, cy, px, py);
		g2.drawString("X=" + _x + ",Y=" + _y + ",B=" + getBearing() + ",S="
				+ _speed, cx, cy + 40);
		
		g2.setColor(Color.YELLOW);
		g2.fillRect(cx - 2, cy - 2, 5, 5);

		g2.setColor(Color.PINK);

		float cos = MathUtil.cos(_bearing);
		float sin = MathUtil.sin(_bearing);
		float _xVector = _speed * sin;
		float _yVector = _speed * cos;

		g2.drawLine(cx, cy, cx, cy - (int) _yVector);
		g2.drawLine(cx, cy, cx + (int) _xVector, cy);
		g2.drawString("X=" + (int) _xVector, cx + (int) _xVector, cy);
		g2.drawString("Y=" + (int) _yVector, cx, cy - (int) _yVector);
	}

	

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#getBearing()
	 */
	public float getBearing() {
		return _bearing;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#getSpeed()
	 */
	public float getSpeed() {
		return _speed;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#getWayPoints()
	 */
	public Point[] getWayPoints() {
		return (Point[]) _ways.toArray(new Point[_ways.size()]);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#execute()
	 */
	public void execute() {
		if (_nextWay == null){
			return;
		}
		if (_isRotating){
			rotate();
			return;
		}
		else {
			moving();
		}
		
	}

	/**
	 * 
	 */
	private void moving() {
		// TODO Auto-generated method stub
		
	}

	/**
	 * 
	 */
	private void rotate() {
		if (! _interpolation.interpolation(1f)){
			_bearing = _interpolation.getValue();
		}
		else {
			
			_isRotating = false;
		}
		
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#addWayPoint(int, int)
	 */
	public void addWayPoint(int x, int y) {
		System.out.println("## Way Point added to " + x + "," + y);
		Point p = new Point(x, y);
		_ways.add(p);
		
		if (_nextWay == null) {
			setWayPoint(x,y);			
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.framework.Unit#setWayPoint(int, int)
	 */
	public void setWayPoint(int x, int y) {
		_ways.clear();
		Point p = new Point(x, y);
		_ways.add(p);
		_nextWay = p;
		_isRotating = true;
		_nextAngle = MathUtil.angle(_x, _y, x, y);
		
		
		_interpolation = new EaseInOutInterpolation();
		_interpolation.setup(_bearing, _nextAngle,60f);
		System.out.println("## Next Angle X1:"+_x+" Y1:"+_y+" X2:"+x+" Y2:"+y+" NA:"+_nextAngle);
		System.out.println("## Interpolation F:"+_bearing+" N:"+_nextAngle);
	}

	public Point getNextWayPoint() {
		return _nextWay;
	}
}