/*
 * Created on 2004. 5. 30.
 *
 * All right reserved to Jisung, Ahn
 */
package net.narusas.game.framework;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;

import javax.swing.JPanel;

import net.narusas.game.eun1.layers.BGLayer;
import net.narusas.game.framework.clock.Clock;
import net.narusas.game.framework.clock.ClockWatcher;
import net.narusas.game.framework.event.EventManager;

/**
 * 
 * 
 * @author Jisung, Ahn
 * 
 *  
 */
public class Game extends JPanel implements ClockWatcher {
	protected Screen _screen;
	protected LayerManager _layerManager;
	protected EventManager _eventManager;
	protected Clock _clock;
	protected Clock _userClock;
	protected boolean _pendingStart;
	private UnitManager _unitManager;

	public Game() {
		init();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.narusas.game.eun.clock.ClockWatcher#tick(net.narusas.game.eun.clock.Clock)
	 */
	public void tick(Clock clock) {
		_unitManager.tick();
		_layerManager.tick();
		_screen.repaint();

	}

	public Clock getSystemClock() {
		return _clock;
	}

	public Clock getUserClock() {
		return _userClock;
	}

	void init() {

		createUnitManager();
		createLayermanager();
		createScreen();
		createClock();
		createEventManager();

	}
	
	public EventManager getEventManager() {
		return _eventManager;
	}

	/**
	 *  
	 */
	private void createUnitManager() {
		_unitManager = new UnitManager(this);

	}

	/**
	 *  
	 */
	private void createEventManager() {
		_eventManager = new EventManager(this);
		this.addKeyListener(_eventManager);
		this.addMouseListener(_eventManager);
	}

	/**
	 *  
	 */
	private void createClock() {
		_clock = new Clock(16);
		_clock.addClockWatcher(this);
		if (_pendingStart) {
			start();
		}

	}

	/**
	 *  
	 */
	private void createScreen() {
		setLayout(new BorderLayout());
		JPanel p = new JPanel();
		p.setLayout(new FlowLayout(FlowLayout.CENTER, 10, 0));
		_screen = new Screen(this, 800, 600);
		setScreen(_screen);
		add(BorderLayout.CENTER, _screen);
		setSize(new Dimension(8000, 600));
		_layerManager.init();

	}

	/**
	 * @param _screen2
	 */
	public void setScreen(Screen screen) {
		_screen = screen;
	}

	public Screen getScreen() {
		return _screen;
	}

	/**
	 *  
	 */
	private void createLayermanager() {
		_layerManager = new LayerManager(this);
		_layerManager.addLayer(new BGLayer(this));

	}

	public void start() {
		if (_clock != null) {
			_clock.start();
		} else {
			_pendingStart = true;
		}
	}

	public void stop() {
		if (_clock != null) {
			_clock.stop();
		}
	}

	public LayerManager getLayerManager() {

		return _layerManager;
	}

	public UnitManager getUnitManager() {
		return _unitManager;
	}

}