/*
 * Created on 2004. 5. 30.
 *
 * All right reserved to Jisung, Ahn
 */
package net.narusas.game.framework;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GraphicsConfiguration;
import java.awt.image.VolatileImage;

/**
 * 
 * 
 * @author Jisung, Ahn
 * 
 *  
 */
public class Screen extends Component {
	Game _game;

	int _width;
	int _height;
	VolatileImage _backBuffer;;

	public Screen(Game game, int w, int h) {
		_game = game;
		_width = w;
		_height = h;
	}

	public Dimension size() {
		return new Dimension(_width, _height);
	}

	public synchronized boolean initOffscreen() {
		if (_backBuffer == null || _width != getWidth()
				|| _height != getHeight()) {
			_width = getWidth();
			_height = getHeight();
			if (_width == 0 || _height == 0) {
				return false;
			}

			_backBuffer = createVolatileImage(_width, _height);

		}
		return true;
	}

	public void resetRestoreVolatileImages() {
		GraphicsConfiguration gc = this.getGraphicsConfiguration();

		int valCode = ((VolatileImage) _backBuffer).validate(gc);
		if (valCode == VolatileImage.IMAGE_INCOMPATIBLE) {
			_backBuffer = createVolatileImage(_width, _height);
		}

	}

	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		if (!initOffscreen()) {
			return;
		}

		// Render the back buffer and copy it to the screen. Loop until
		// the buffer contents are correct (this will only loop when/if
		// surface are lost, causing content loss on volatile images).
		try {
			do {
				resetRestoreVolatileImages();
				Graphics2D backG2 = (Graphics2D) _backBuffer.getGraphics();
				drawLaysers(backG2);
				backG2.dispose();
				g.drawImage(_backBuffer, 0, 0, this);
			} while (_backBuffer.contentsLost());
		} catch (Exception e) {
			System.out.println("Exception during paint: " + e);
		}
	}

	/**
	 * @param backG2
	 */
	private void drawLaysers(Graphics2D backG2) {
		for (int i = 0; i < _game._layerManager.size(); i++) {
			Layer layer = _game._layerManager.layer(i);
			layer.paint(backG2);
		}

	}
}