package net.narusas.game.tactical2;

import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.PathIterator;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.util.ArrayList;

/**
 * @author Administrator
 * 
 * To change this generated comment edit the template variable "typecomment":
 * Window>Preferences>Java>Templates. To enable and disable the creation of type
 * comments go to Window>Preferences>Java>Code Generation.
 */
public class CollisionEngine {
	final static int DEFAULT_POINT_POOL_SIZE = 20;
	final static int DEFAULT_LIST_POOL_SIZE = 20;

	ArrayList pointPool;
	ArrayList listPool;

	int pointPoolUsageCount;
	int pointPoolPosition;

	int listPoolUsageCount;
	int listPoolPosition;

	public static void main(String[] args) {

		new CollisionEngine().test();

	}

	public CollisionEngine() {

		pointPool = new ArrayList();
		listPool = new ArrayList();

		for (int i = 0; i < DEFAULT_LIST_POOL_SIZE; i++) {

			listPool.add(new ArrayList());

		}

		for (int i = 0; i < DEFAULT_POINT_POOL_SIZE; i++) {

			pointPool.add(new Point2D.Double());

		}

	}

	public boolean isInside(Shape source, Point2D hitPoint) {

		Rectangle2D bound = source.getBounds2D();
		if (!bound.contains(hitPoint.getX(), hitPoint.getY()))
			return false;

		Point2D[] list = getPointsOnShape(source);

		boolean flag = isInside(list, hitPoint);

		return flag;

	}

	public boolean isCollided(Shape source, Shape hitShape) {

		return true;

	}

	boolean isInside(Point2D[] vertexs, Point2D hitPoint) {

		int edge = 0;
		int first = 0;
		int next = 0;
		int quad = 0;
		int nextQuad = 0;
		int delta = 0;
		int total = 0;

		int max = vertexs.length - 1;

		quad = whichQuad(vertexs[edge], hitPoint);

		for (int i = 0; i <= max; i++) {

			next++;
			if (next <= max)
				nextQuad = whichQuad(vertexs[next], hitPoint);
			else
				nextQuad = whichQuad(vertexs[0], hitPoint);
			delta = nextQuad - quad;

			switch (delta) {

				case 2 :
				case -2 :
					if (next <= max) {
						if (isXIntercepted(vertexs[edge], vertexs[next],
								hitPoint.getY())) {
							delta = -delta;
						}
					} else {
						if (isXIntercepted(vertexs[edge], vertexs[0], hitPoint
								.getY())) {
							delta = -delta;
						}
					}
					break;

				case 3 :
					delta = -1;
					break;
				case -3 :
					delta = 1;
					break;
			}

			total += delta;
			quad = nextQuad;
			edge = next;

		}

		if ((total == 4) || (total == -4))
			return true;

		return false;

	}

	Point2D[] getPointsOnShape(Shape shape) {

		PathIterator paths = shape.getPathIterator(null);

		ArrayList list = new ArrayList();
		double[] pos = new double[2];
		int ct = 0;
		while (true) {
			if (paths.isDone())
				break;

			int type = paths.currentSegment(pos);

			if (type == PathIterator.SEG_CLOSE) {
				break;
			}
			Point2D p = new Point2D.Double(pos[0], pos[1]);
			list.add(p);
			paths.next();
			ct++;
		}
		return (Point2D[]) list.toArray(new Point2D[0]);

	}

	public Line2D[] getLinesOnShape(Shape shape) {
		Point2D[] pts = getPointsOnShape(shape);
		if (pts.length < 2) {
			return null;
		}
		Line2D[] lines = new Line2D[pts.length];
		for (int i = 0; i < pts.length - 1; i++) {
			lines[i] = new Line2D.Double(pts[i], pts[i + 1]);
		}
		lines[pts.length - 1] = new Line2D.Double(pts[pts.length - 1], pts[0]);
		return lines;
	}

	public Point2D[] getIntersectedLine(Shape shape, Line2D line) {
		Line2D[] src = getLinesOnShape(shape);
		ArrayList list = new ArrayList();
		for (int i = 0; i < src.length; i++) {
			Point2D pt = getCrossPoint(src[i], line);
			if (pt != null) {
				list.add(pt);
			}
		}
		return (Point2D[]) list.toArray(new Point2D[0]);

	}

	public static int whichQuad(Point2D vertex, Point2D hitPos) {

		return ((vertex.getX() < hitPos.getX()) ? ((vertex.getY() > hitPos
				.getY()) ? 4 : 1) : ((vertex.getY() > hitPos.getY()) ? 3 : 2));

	}

	public static boolean isXIntercepted(Point2D point1, Point2D point2,
			double hitY) {

		double r = (point2.getX() - (((point2.getY() - hitY) * (point1.getX() - point2
				.getX())) / (point1.getY() - point2.getY())));
		if (((int) r) == 0)
			return false;
		return true;

	}

	public boolean checkBouncingBoxIntersect(Line2D src, Line2D target) {
		double d = src.getX1() >= src.getX2() ? src.getX2() : src.getX1();
		double d1 = src.getY1() >= src.getY2() ? src.getY2() : src.getY1();
		double d2 = src.getX1() < src.getX2() ? src.getX2() : src.getX1();
		double d3 = src.getY1() < src.getY2() ? src.getY2() : src.getY1();

		double d4 = target.getX1() >= target.getX2() ? target.getX2() : target
				.getX1();
		double d5 = target.getY1() >= target.getY2() ? target.getY2() : target
				.getY1();
		double d6 = target.getX1() < target.getX2() ? target.getX2() : target
				.getX1();
		double d7 = target.getY1() < target.getY2() ? target.getY2() : target
				.getY1();

		return d2 >= d4 && d6 >= d && d3 >= d5 && d7 >= d1;

	}

	public int CrossMagnitude(int i, int j, int k, int l) {
		return i * l - k * j;
	}

	public double CrossMagnitude(double a, double b, double c, double d) {
		return a * d - b * c;
	}

	public boolean isIntersectLine(Line2D src, Line2D target) {
		if (!checkBouncingBoxIntersect(src, target)) {
			return false;
		}

		double d = CrossMagnitude(target.getX1() - src.getX1(), target.getY1()
				- src.getY1(), src.getX2() - src.getX1(), src.getY2()
				- src.getY1()) < 0.0D ? -1 : 1;
		double d1 = CrossMagnitude(target.getX2() - src.getX1(), target.getY2()
				- src.getY1(), src.getX2() - src.getX1(), src.getY2()
				- src.getY1()) < 0.0D ? -1 : 1;
		double d2 = CrossMagnitude(src.getX1() - target.getX1(), src.getY1()
				- target.getY1(), target.getX2() - target.getX1(), target
				.getY2()
				- target.getY1()) < 0.0D ? -1 : 1;
		double d3 = CrossMagnitude(src.getX2() - target.getX1(), src.getY2()
				- target.getY1(), target.getX2() - target.getX1(), target
				.getY2()
				- target.getY1()) < 0.0D ? -1 : 1;
		return d != d1 && d2 != d3;
	}

	public Point2D getCrossPoint(Line2D src, Line2D target) {
		return getCrossPoint1(src, target);
	}

	private double getSlope(Line2D src) {
		if (src.getX2() - src.getX1() == 0.0D)
			return 99999999999999.906D;
		else
			return (src.getY2() - src.getY1()) / (src.getX2() - src.getX1());

	}
	private Point2D getCrossPoint2(Line2D src, Line2D target) {
		if (!isIntersectLine(src, target))
			return null;
		double d = getSlope(src);
		double d1 = getSlope(target);
		double d2 = CrossMagnitude(-d, 1.0D, -d1, 1.0D);
		double d3 = CrossMagnitude(src.getY1() - d * src.getX1(), 1.0D, target
				.getY1()
				- d1 * target.getX1(), 1.0D);
		double d4 = CrossMagnitude(-d, src.getY1() - d * src.getX1(), -d1,
				target.getY1() - d1 * target.getX1());
		double d5 = d3 / d2;
		double d6 = d4 / d2;
		if (Math.abs(d5 - (double) (int) d5) < 9.9999999999999995E-008D)
			d5 = (int) d5;
		else if (Math.abs(d5 - (double) (int) (d5 + 0.5D)) < 9.9999999999999995E-008D)
			d5 = (int) (d5 + 0.5D);
		if (Math.abs(d6 - (double) (int) d6) < 9.9999999999999995E-008D)
			d6 = (int) d6;
		else if (Math.abs(d6 - (double) (int) (d6 + 0.5D)) < 9.9999999999999995E-008D)
			d6 = (int) (d6 + 0.5D);
		Point2D point = new Point2D.Double(d5, d6);
		return point;

	}

	private Point2D getCrossPoint1(Line2D src, Line2D target) {
		//        Point2D[] srcPoint = getPointsOnShape(src);
		//        Point2D[] targetPoint = getPointsOnShape(target);

		int MAXITER = 50;
		double EPS = 10e-10;
		double t1, s1, del_t = 0, del_s = 0, Fts = 0, Gts = 0;

		t1 = s1 = 0.0;
		//        double srcXGap = srcPoint[1].getX() - srcPoint[0].getX();
		//        double srcYGap = srcPoint[1].getY() - srcPoint[0].getY();
		//
		//        double targetXGap = targetPoint[1].getX() - targetPoint[0].getX();
		//        double targetYGap = targetPoint[1].getY() - targetPoint[0].getY();

		double x0 = src.getX1();
		double y0 = src.getY1();

		double x1 = src.getX2();
		double y1 = src.getY2();

		double x2 = target.getX1();
		double y2 = target.getY1();

		double x3 = target.getX2();
		double y3 = target.getY2();

		double a1, b1, c1, // constants of linear equations
		a2, b2, c2, det_inv, // the inverse of the determinant of the
							 // coefficient matrix
		m1, m2; // the slopes of each line

		if ((x1 - x0) != 0)
			m1 = (y1 - y0) / (x1 - x0);
		else
			m1 = (double) 1e+10; // close enough to infinity

		if ((x3 - x2) != 0)
			m2 = (y3 - y2) / (x3 - x2);
		else
			m2 = (double) 1e+10; // close enough to infinity

		// compute constants

		a1 = m1;
		a2 = m2;

		b1 = -1;
		b2 = -1;

		c1 = (y0 - m1 * x0);
		c2 = (y2 - m2 * x2);

		// compute the inverse of the determinate

		det_inv = 1 / (a1 * b2 - a2 * b1);

		// use Kramers rule to compute xi and yi

		double x = ((b1 * c2 - b2 * c1) * det_inv);
		double y = ((a2 * c1 - a1 * c2) * det_inv);

		double maxX = Math.max(src.getX1(), src.getX2());
		double minX = Math.min(src.getX1(), src.getX2());
		double maxY = Math.max(src.getY1(), src.getY2());
		double minY = Math.min(src.getY1(), src.getY2());

		if (x > (maxX + .1) || x < (minX - .1) || y > (maxY + .1)
				|| y < (minY - .1)) {
			return null;
		}

		return new Point2D.Double(x, y);

	}

	public void test() {
		/*
		 * Rectangle rect = new Rectangle(0,0,500,450); double p = 0; Point2D
		 * point = new Point2D.Double(240,240); int in = 0; int out = 0; long
		 * start = System.currentTimeMillis(); for ( int i=0; i < 18000; i++) {
		 * if ( isInside(rect, point) ) { in++; } else { out++; }
		 * point.setLocation(p+i, p+i); } long end = System.currentTimeMillis();
		 * System.out.println("Inside:"+in); System.out.println("Outside:"+out);
		 * System.out.println("Cal:"+(end-start));
		 */
		Line2D src = new Line2D.Double((double) 0, (double) 0, (double) 100,
				(double) 100);
		Line2D target = new Line2D.Double((double) 80, (double) 65,
				(double) 120, (double) 65);
		Point2D crossPoint = getCrossPoint(src, target);
		if (crossPoint == null) {
			System.out.println("Not crossed");
		} else {
			System.out.println("X:" + crossPoint.getX() + " Y:"
					+ crossPoint.getY());
		}

	}

	public double getLineLength(Line2D src) {
		return Math.sqrt((src.getX1() - src.getX2())
				* (src.getX1() - src.getX2()) + (src.getY1() - src.getY2())
				* (src.getY1() - src.getY2()));
	}

} // end class
